@@ -1,54 +1,25 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2011-2015 Raphaël Barrois
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
+# Copyright: See the LICENSE file.
+
 """Tests for factory_boy/Django interactions."""
 
 import os
 from .compat import is_python2, unittest, mock
 
 
-try:
-    import django
-except ImportError:  # pragma: no cover
-    django = None
+import django
 
 # Setup Django as soon as possible
-if django is not None:
-    os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'tests.djapp.settings')
-
-    if django.VERSION >= (1, 7, 0):
-        django.setup()
-    from django import test as django_test
-    from django.conf import settings
-    from django.db import models as django_models
-    if django.VERSION <= (1, 8, 0):
-        from django.test.simple import DjangoTestSuiteRunner
-    else:
-        from django.test.runner import DiscoverRunner as DjangoTestSuiteRunner
-    from django.test import utils as django_test_utils
-    from django.db.models import signals
-    from .djapp import models
-
-else:
-    django_test = unittest
+os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'tests.djapp.settings')
 
+django.setup()
+from django import test as django_test
+from django.conf import settings
+from django.db import models as django_models
+from django.test.runner import DiscoverRunner as DjangoTestSuiteRunner
+from django.test import utils as django_test_utils
+from django.db.models import signals
+from .djapp import models
 
 
 try:
@@ -67,15 +38,12 @@ import factory.django
 from factory.compat import BytesIO
 
 from . import testdata
-from . import tools
 
 
 test_state = {}
 
 
 def setUpModule():
-    if django is None:  # pragma: no cover
-        raise unittest.SkipTest("Django not installed")
     django_test_utils.setup_test_environment()
     runner = DjangoTestSuiteRunner()
     runner_state = runner.setup_databases()
@@ -86,99 +54,95 @@ def setUpModule():
 
 
 def tearDownModule():
-    if django is None:  # pragma: no cover
-        return
     runner = test_state['runner']
     runner_state = test_state['runner_state']
     runner.teardown_databases(runner_state)
     django_test_utils.teardown_test_environment()
 
 
-if django is not None:
-    class StandardFactory(factory.django.DjangoModelFactory):
-        class Meta:
-            model = models.StandardModel
+class StandardFactory(factory.django.DjangoModelFactory):
+    class Meta:
+        model = models.StandardModel
 
-        foo = factory.Sequence(lambda n: "foo%d" % n)
+    foo = factory.Sequence(lambda n: "foo%d" % n)
 
 
-    class StandardFactoryWithPKField(factory.django.DjangoModelFactory):
-        class Meta:
-            model = models.StandardModel
-            django_get_or_create = ('pk',)
+class StandardFactoryWithPKField(factory.django.DjangoModelFactory):
+    class Meta:
+        model = models.StandardModel
+        django_get_or_create = ('pk',)
 
-        foo = factory.Sequence(lambda n: "foo%d" % n)
-        pk = None
+    foo = factory.Sequence(lambda n: "foo%d" % n)
+    pk = None
 
 
-    class NonIntegerPkFactory(factory.django.DjangoModelFactory):
-        class Meta:
-            model = models.NonIntegerPk
+class NonIntegerPkFactory(factory.django.DjangoModelFactory):
+    class Meta:
+        model = models.NonIntegerPk
 
-        foo = factory.Sequence(lambda n: "foo%d" % n)
-        bar = ''
+    foo = factory.Sequence(lambda n: "foo%d" % n)
+    bar = ''
 
 
-    class MultifieldModelFactory(factory.django.DjangoModelFactory):
-        class Meta:
-            model = models.MultifieldModel
-            django_get_or_create = ['slug']
+class MultifieldModelFactory(factory.django.DjangoModelFactory):
+    class Meta:
+        model = models.MultifieldModel
+        django_get_or_create = ['slug']
 
-        text = factory.Faker('text')
+    text = factory.Faker('text')
 
 
-    class AbstractBaseFactory(factory.django.DjangoModelFactory):
-        class Meta:
-            model = models.AbstractBase
-            abstract = True
+class AbstractBaseFactory(factory.django.DjangoModelFactory):
+    class Meta:
+        model = models.AbstractBase
+        abstract = True
 
-        foo = factory.Sequence(lambda n: "foo%d" % n)
+    foo = factory.Sequence(lambda n: "foo%d" % n)
 
 
-    class ConcreteSonFactory(AbstractBaseFactory):
-        class Meta:
-            model = models.ConcreteSon
+class ConcreteSonFactory(AbstractBaseFactory):
+    class Meta:
+        model = models.ConcreteSon
 
 
-    class AbstractSonFactory(AbstractBaseFactory):
-        class Meta:
-            model = models.AbstractSon
+class AbstractSonFactory(AbstractBaseFactory):
+    class Meta:
+        model = models.AbstractSon
 
 
-    class ConcreteGrandSonFactory(AbstractBaseFactory):
-        class Meta:
-            model = models.ConcreteGrandSon
+class ConcreteGrandSonFactory(AbstractBaseFactory):
+    class Meta:
+        model = models.ConcreteGrandSon
 
 
-    class WithFileFactory(factory.django.DjangoModelFactory):
-        class Meta:
-            model = models.WithFile
+class WithFileFactory(factory.django.DjangoModelFactory):
+    class Meta:
+        model = models.WithFile
 
-        if django is not None:
-            afile = factory.django.FileField()
+    if django is not None:
+        afile = factory.django.FileField()
 
 
-    class WithImageFactory(factory.django.DjangoModelFactory):
-        class Meta:
-            model = models.WithImage
+class WithImageFactory(factory.django.DjangoModelFactory):
+    class Meta:
+        model = models.WithImage
 
-        if django is not None:
-            animage = factory.django.ImageField()
+    if django is not None:
+        animage = factory.django.ImageField()
 
 
-    class WithSignalsFactory(factory.django.DjangoModelFactory):
-        class Meta:
-            model = models.WithSignals
+class WithSignalsFactory(factory.django.DjangoModelFactory):
+    class Meta:
+        model = models.WithSignals
 
 
-    class WithCustomManagerFactory(factory.django.DjangoModelFactory):
-        class Meta:
-            model = models.WithCustomManager
+class WithCustomManagerFactory(factory.django.DjangoModelFactory):
+    class Meta:
+        model = models.WithCustomManager
 
-        foo = factory.Sequence(lambda n: "foo%d" % n)
+    foo = factory.Sequence(lambda n: "foo%d" % n)
 
 
-@unittest.skipIf(django is None, "Django not installed.")
 class ModelTests(django_test.TestCase):
     def test_unset_model(self):
         class UnsetModelFactory(factory.django.DjangoModelFactory):
@@ -197,7 +161,6 @@ class ModelTests(django_test.TestCase):
         self.assertEqual(obj, models.StandardModel.objects.using('replica').get())
 
 
-@unittest.skipIf(django is None, "Django not installed.")
 class DjangoPkSequenceTestCase(django_test.TestCase):
     def setUp(self):
         super(DjangoPkSequenceTestCase, self).setUp()
@@ -234,7 +197,6 @@ class DjangoPkSequenceTestCase(django_test.TestCase):
         self.assertEqual(11, std2.pk)
 
 
-@unittest.skipIf(django is None, "Django not installed.")
 class DjangoGetOrCreateTests(django_test.TestCase):
     def test_simple_call(self):
         obj1 = MultifieldModelFactory(slug='slug1')
@@ -257,7 +219,6 @@ class DjangoGetOrCreateTests(django_test.TestCase):
         self.assertEqual(2, models.MultifieldModel.objects.count())
 
 
-@unittest.skipIf(django is None, "Django not installed.")
 class DjangoPkForceTestCase(django_test.TestCase):
     def setUp(self):
         super(DjangoPkForceTestCase, self).setUp()
@@ -282,7 +243,6 @@ class DjangoPkForceTestCase(django_test.TestCase):
         self.assertEqual('bar', std2.foo)
 
 
-@unittest.skipIf(django is None, "Django not installed.")
 class DjangoModelLoadingTestCase(django_test.TestCase):
     """Tests class Meta:
      model = 'app.Model' pattern."""
@@ -292,7 +252,7 @@ class DjangoModelLoadingTestCase(django_test.TestCase):
             class Meta:
                 model = 'djapp.StandardModel'
 
-        self.assertEqual(models.StandardModel, ExampleFactory._get_model_class())
+        self.assertEqual(models.StandardModel, ExampleFactory._meta.get_model_class())
 
     def test_building(self):
         class ExampleFactory(factory.DjangoModelFactory):
@@ -332,8 +292,6 @@ class DjangoModelLoadingTestCase(django_test.TestCase):
             class Meta:
                 model = 'djapp.StandardSon'
 
-        self.assertEqual(models.StandardSon, Example2Factory._get_model_class())
-
         e1 = ExampleFactory.build()
         e2 = Example2Factory.build()
         e3 = ExampleFactory.build()
@@ -345,7 +303,6 @@ class DjangoModelLoadingTestCase(django_test.TestCase):
         self.assertEqual(2, e3.foo)
 
 
-@unittest.skipIf(django is None, "Django not installed.")
 class DjangoNonIntegerPkTestCase(django_test.TestCase):
     def setUp(self):
         super(DjangoNonIntegerPkTestCase, self).setUp()
@@ -382,7 +339,6 @@ class DjangoNonIntegerPkTestCase(django_test.TestCase):
         self.assertEqual('foo0', nonint2.pk)
 
 
-@unittest.skipIf(django is None, "Django not installed.")
 class DjangoAbstractBaseSequenceTestCase(django_test.TestCase):
     def test_auto_sequence(self):
         """The sequence of the concrete son of an abstract model should be autonomous."""
@@ -411,7 +367,6 @@ class DjangoAbstractBaseSequenceTestCase(django_test.TestCase):
         self.assertEqual("foo0", obj.foo)
 
 
-@unittest.skipIf(django is None, "Django not installed.")
 class DjangoRelatedFieldTestCase(django_test.TestCase):
 
     @classmethod
@@ -420,24 +375,75 @@ class DjangoRelatedFieldTestCase(django_test.TestCase):
         class PointedFactory(factory.django.DjangoModelFactory):
             class Meta:
                 model = models.PointedModel
-            foo = 'ahah'
+            foo = 'foo'
 
         class PointerFactory(factory.django.DjangoModelFactory):
             class Meta:
-                model = models.PointingModel
-            pointed = factory.SubFactory(PointedFactory, foo='hihi')
-            foo = 'bar'
+                model = models.PointerModel
+            bar = 'bar'
+            pointed = factory.SubFactory(PointedFactory, foo='new_foo')
+
+        class PointedRelatedFactory(PointedFactory):
+            pointer = factory.RelatedFactory(PointerFactory, 'pointed')
+
+        class PointerExtraFactory(PointerFactory):
+            pointed__foo = 'extra_new_foo'
+
+        class PointedRelatedExtraFactory(PointedRelatedFactory):
+            pointer__bar = 'extra_new_bar'
 
         cls.PointedFactory = PointedFactory
         cls.PointerFactory = PointerFactory
+        cls.PointedRelatedFactory = PointedRelatedFactory
+        cls.PointerExtraFactory = PointerExtraFactory
+        cls.PointedRelatedExtraFactory = PointedRelatedExtraFactory
 
-    def test_direct_related_create(self):
-        ptr = self.PointerFactory()
-        self.assertEqual('hihi', ptr.pointed.foo)
-        self.assertEqual(ptr.pointed, models.PointedModel.objects.get())
+    def test_create_pointed(self):
+        pointed = self.PointedFactory()
+        self.assertEqual(pointed, models.PointedModel.objects.get())
+        self.assertEqual(pointed.foo, 'foo')
+
+    def test_create_pointer(self):
+        pointer = self.PointerFactory()
+        self.assertEqual(pointer.pointed, models.PointedModel.objects.get())
+        self.assertEqual(pointer.pointed.foo, 'new_foo')
+
+    def test_create_pointer_with_deep_context(self):
+        pointer = self.PointerFactory(pointed__foo='new_new_foo')
+        self.assertEqual(pointer, models.PointerModel.objects.get())
+        self.assertEqual(pointer.bar, 'bar')
+        self.assertEqual(pointer.pointed, models.PointedModel.objects.get())
+        self.assertEqual(pointer.pointed.foo, 'new_new_foo')
+
+    def test_create_pointed_related(self):
+        pointed = self.PointedRelatedFactory()
+        self.assertEqual(pointed, models.PointedModel.objects.get())
+        self.assertEqual(pointed.foo, 'foo')
+        self.assertEqual(pointed.pointer, models.PointerModel.objects.get())
+        self.assertEqual(pointed.pointer.bar, 'bar')
+
+    def test_create_pointed_related_with_deep_context(self):
+        pointed = self.PointedRelatedFactory(pointer__bar='new_new_bar')
+        self.assertEqual(pointed, models.PointedModel.objects.get())
+        self.assertEqual(pointed.foo, 'foo')
+        self.assertEqual(pointed.pointer, models.PointerModel.objects.get())
+        self.assertEqual(pointed.pointer.bar, 'new_new_bar')
+
+    def test_create_pointer_extra(self):
+        pointer = self.PointerExtraFactory()
+        self.assertEqual(pointer, models.PointerModel.objects.get())
+        self.assertEqual(pointer.bar, 'bar')
+        self.assertEqual(pointer.pointed, models.PointedModel.objects.get())
+        self.assertEqual(pointer.pointed.foo, 'extra_new_foo')
+
+    def test_create_pointed_related_extra(self):
+        pointed = self.PointedRelatedExtraFactory()
+        self.assertEqual(pointed, models.PointedModel.objects.get())
+        self.assertEqual(pointed.foo, 'foo')
+        self.assertEqual(pointed.pointer, models.PointerModel.objects.get())
+        self.assertEqual(pointed.pointer.bar, 'extra_new_bar')
 
 
-@unittest.skipIf(django is None, "Django not installed.")
 class DjangoFileFieldTestCase(unittest.TestCase):
 
     def tearDown(self):
@@ -548,7 +554,59 @@ class DjangoFileFieldTestCase(unittest.TestCase):
         self.assertFalse(o.afile)
 
 
-@unittest.skipIf(django is None, "Django not installed.")
+class DjangoParamsTestCase(django_test.TestCase):
+
+    def test_undeclared_fields(self):
+        class WithDefaultValueFactory(factory.django.DjangoModelFactory):
+            class Meta:
+                model = models.WithDefaultValue
+
+            class Params:
+                with_bar = factory.Trait(
+                    foo='bar',
+                )
+
+        o = WithDefaultValueFactory()
+        self.assertEqual('', o.foo)
+
+    def test_pointing_with_traits_using_same_name(self):
+        class PointedFactory(factory.django.DjangoModelFactory):
+            class Meta:
+                model = models.PointedModel
+
+            class Params:
+                with_bar = factory.Trait(
+                    foo='bar',
+                )
+
+        class PointerFactory(factory.django.DjangoModelFactory):
+            class Meta:
+                model = models.PointerModel
+            pointed = factory.SubFactory(PointedFactory)
+
+            class Params:
+                with_bar = factory.Trait(
+                    bar='bar',
+                    pointed__with_bar=True,
+                )
+
+        o = PointerFactory(with_bar=True)
+        self.assertEqual('bar', o.bar)
+        self.assertEqual('bar', o.pointed.foo)
+
+
+class DjangoFakerTestCase(unittest.TestCase):
+    def test_random(self):
+        class StandardModelFactory(factory.django.DjangoModelFactory):
+            class Meta:
+                model = models.StandardModel
+            foo = factory.Faker('pystr')
+
+        o1 = StandardModelFactory()
+        o2 = StandardModelFactory()
+        self.assertNotEqual(o1.foo, o2.foo)
+
+
 @unittest.skipIf(Image is None, "PIL not installed.")
 class DjangoImageFieldTestCase(unittest.TestCase):
 
@@ -711,7 +769,6 @@ class DjangoImageFieldTestCase(unittest.TestCase):
         self.assertEqual(32, i.height)
 
 
-@unittest.skipIf(django is None, "Django not installed.")
 class PreventSignalsTestCase(unittest.TestCase):
     def setUp(self):
         self.handlers = mock.MagicMock()
@@ -832,7 +889,6 @@ class PreventSignalsTestCase(unittest.TestCase):
         self.assertSignalsReactivated()
 
 
-@unittest.skipIf(django is None, "Django not installed.")
 class DjangoCustomManagerTestCase(unittest.TestCase):
 
     def test_extra_args(self):
