@@ -1,26 +1,11 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2011-2015 Raphaël Barrois
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
+# Copyright: See the LICENSE file.
+
 """Tests using factory."""
 
 
+import collections
+import datetime
 import functools
 import os
 import sys
@@ -30,7 +15,7 @@ import factory
 from factory import errors
 
 from .compat import is_python2, unittest
-from . import tools
+from . import utils
 
 
 class TestObject(object):
@@ -51,6 +36,23 @@ class TestObject(object):
         )
 
 
+class Dummy(object):
+    def __init__(self, **kwargs):
+        self._fields = set(kwargs)
+        for k, v in kwargs.items():
+            setattr(self, k, v)
+
+    @property
+    def as_dict(self):
+        return {field: getattr(self, field) for field in self._fields}
+
+    def __repr__(self):
+        return '%s(%s)' % (
+            self.__class__.__name__,
+            ', '.join('%s=%r' % pair for pair in sorted(self.as_dict.items()))
+        )
+
+
 class FakeModel(object):
     @classmethod
     def create(cls, **kwargs):
@@ -1115,6 +1117,89 @@ class KwargAdjustTestCase(unittest.TestCase):
         self.assertEqual(42, obj.attributes)
 
 
+class MaybeTestCase(unittest.TestCase):
+    def test_simple_maybe(self):
+        class DummyFactory(factory.Factory):
+            class Meta:
+                model = Dummy
+
+            # Undeclared: key = None
+            both = factory.Maybe('key', 1, 2)
+            yes = factory.Maybe('key', 1)
+            no = factory.Maybe('key', no_declaration=2)
+            none = factory.Maybe('key')
+
+        obj_default = DummyFactory.build()
+        obj_true = DummyFactory.build(key=True)
+        obj_false = DummyFactory.build(key=False)
+
+        self.assertEqual(dict(both=2, no=2), obj_default.as_dict)
+        self.assertEqual(dict(key=True, both=1, yes=1), obj_true.as_dict)
+        self.assertEqual(dict(key=False, both=2, no=2), obj_false.as_dict)
+
+    def test_declarations(self):
+        class DummyFactory(factory.Factory):
+            class Meta:
+                model = Dummy
+
+            a = 0
+            b = 1
+
+            # biggest = 'b' if .b > .a else 'a'
+            biggest = factory.Maybe(factory.LazyAttribute(lambda o: o.a < o.b), 'b', 'a')
+            # max_value = .b if .b > .a else .a = max(.a, .b)
+            max_value = factory.Maybe(factory.LazyAttribute(lambda o: o.a < o.b), factory.SelfAttribute('b'), factory.SelfAttribute('a'))
+
+        obj_ordered = DummyFactory.build(a=1, b=2)
+        obj_equal = DummyFactory.build(a=3, b=3)
+        obj_reverse = DummyFactory.build(a=5, b=4)
+
+        self.assertEqual(dict(a=1, b=2, biggest='b', max_value=2, ), obj_ordered.as_dict)
+        self.assertEqual(dict(a=3, b=3, biggest='a', max_value=3, ), obj_equal.as_dict)
+        self.assertEqual(dict(a=5, b=4, biggest='a', max_value=5, ), obj_reverse.as_dict)
+
+    def test_post_generation(self):
+
+        # Helpers
+        @factory.post_generation
+        def square(obj, *args, **kwargs):
+            obj.value *= obj.value
+
+        @factory.post_generation
+        def quintuple(obj, *args, **kwargs):
+            obj.value *= 5
+
+        @factory.post_generation
+        def double(obj, *args, **kwargs):
+            obj.value *= 2
+
+        @factory.post_generation
+        def decrement(obj, *args, **kwargs):
+            obj.value -= 1
+
+
+        class DummyFactory(factory.Factory):
+            class Meta:
+                model = Dummy
+
+            value = 0
+            square_it = factory.Maybe('square', square)
+            quintuple_it = factory.Maybe('quintuple', quintuple)
+            adjust_nums = factory.Maybe(
+                factory.LazyAttribute(lambda o: o.value % 2 == 0),
+                double,
+                decrement,
+            )
+
+        obj_untouched = DummyFactory.build(value=4)
+        obj_squared = DummyFactory.build(value=5, square=True)
+        obj_combined = DummyFactory.build(value=6, square=True, quintuple=True)
+
+        self.assertEqual(4 * 2, obj_untouched.value)
+        self.assertEqual(5 ** 2 - 1, obj_squared.value)
+        self.assertEqual(6 ** 2 * 5 * 2, obj_combined.value)
+
+
 class TraitTestCase(unittest.TestCase):
     def test_traits(self):
         class TestObjectFactory(factory.Factory):
@@ -1145,6 +1230,28 @@ class TraitTestCase(unittest.TestCase):
         self.assertEqual(obj5.as_dict(),
             dict(one=True, two=True, three=True, four=None, five=True))
 
+    def test_post_generation_traits(self):
+        @factory.post_generation
+        def compute(obj, _create, _value, power=2, **kwargs):
+            obj.value = obj.value ** power
+
+        class DummyFactory(factory.Factory):
+            class Meta:
+                model = Dummy
+
+            value = 3
+            class Params:
+                exponentiate = factory.Trait(apply_exponent=compute)
+
+        base = DummyFactory.build()
+        self.assertEqual(dict(value=3), base.as_dict)
+
+        exp = DummyFactory.build(exponentiate=True)
+        self.assertEqual(dict(value=9), exp.as_dict)
+
+        higher = DummyFactory.build(exponentiate=True, apply_exponent__power=4)
+        self.assertEqual(dict(value=81), higher.as_dict)
+
     def test_traits_inheritance(self):
         """A trait can be set in an inherited class."""
         class TestObjectFactory(factory.Factory):
@@ -1197,6 +1304,7 @@ class TraitTestCase(unittest.TestCase):
                 even = factory.Trait(two=True, four=True)
                 odd = factory.Trait(one=True, three=True, five=True)
                 full = factory.Trait(even=True, odd=True)
+                override = factory.Trait(even=True, two=False)
 
         # Setting "full" should enable all fields.
         obj = TestObjectFactory(full=True)
@@ -1216,6 +1324,11 @@ class TraitTestCase(unittest.TestCase):
         self.assertEqual(obj3.as_dict(),
             dict(one=True, two=None, three=True, four=None, five=True))
 
+        # Setting override should override two and set it to False
+        obj = TestObjectFactory(override=True)
+        self.assertEqual(obj.as_dict(),
+            dict(one=None, two=False, three=None, four=True, five=None))
+
     def test_prevent_cyclic_traits(self):
 
         with self.assertRaises(errors.CyclicDefinitionError):
@@ -1227,6 +1340,57 @@ class TraitTestCase(unittest.TestCase):
                     a = factory.Trait(b=True, one=True)
                     b = factory.Trait(a=True, two=True)
 
+    def test_deep_traits(self):
+        class TestObjectFactory(factory.Factory):
+            class Meta:
+                model = TestObject
+
+        class WrapperFactory(factory.Factory):
+            class Meta:
+                model = TestObject
+
+            class Params:
+                deep_one = factory.Trait(
+                    one=1,
+                    two__one=2,
+                )
+
+            two = factory.SubFactory(TestObjectFactory)
+
+        wrapper = WrapperFactory(deep_one=True)
+        self.assertEqual(1, wrapper.one)
+        self.assertEqual(2, wrapper.two.one)
+
+    def test_traits_and_postgeneration(self):
+        """A trait parameter should be resolved before post_generation hooks.
+
+        See https://github.com/FactoryBoy/factory_boy/issues/466.
+        """
+        PRICES = {}
+
+        Pizza = collections.namedtuple('Pizza', ['style', 'toppings'])
+
+        class PizzaFactory(factory.Factory):
+            class Meta:
+                model = Pizza
+
+            class Params:
+                fancy = factory.Trait(
+                    toppings=['eggs', 'ham', 'extra_cheese'],
+                    pricing__extra=10,
+                )
+
+            pricing__extra = 0
+            toppings = ['tomato', 'cheese']
+            style = 'margharita'
+
+            @factory.post_generation
+            def pricing(self, create, extracted, base_price=5, extra=0, **kwargs):
+                PRICES[base_price + extra] = self
+
+        p = PizzaFactory.build()
+        self.assertEqual({5: p}, PRICES)
+
 
 class SubFactoryTestCase(unittest.TestCase):
     def test_sub_factory(self):
@@ -1340,6 +1504,32 @@ class SubFactoryTestCase(unittest.TestCase):
         self.assertEqual(wrapping.wrapped.three, 3)
         self.assertEqual(wrapping.wrapped.four, 4)
 
+    def test_sub_factory_deep_overrides(self):
+        Author = collections.namedtuple('Author', ['name', 'country'])
+        Book = collections.namedtuple('Book', ['title', 'author'])
+        Chapter = collections.namedtuple('Chapter', ['book', 'number'])
+
+        class AuthorFactory(factory.Factory):
+            class Meta:
+                model = Author
+            name = "John"
+            country = 'XX'
+
+        class BookFactory(factory.Factory):
+            class Meta:
+                model = Book
+            title = "The mighty adventures of nobody."
+            author = factory.SubFactory(AuthorFactory)
+
+        class ChapterFactory(factory.Factory):
+            class Meta:
+                model = Chapter
+            book = factory.SubFactory(BookFactory)
+            number = factory.Sequence(lambda n: n)
+            book__author__country = factory.LazyAttribute(lambda o: 'FR')
+
+        c = ChapterFactory()
+
     def test_nested_sub_factory(self):
         """Test nested sub-factories."""
 
@@ -1428,6 +1618,54 @@ class SubFactoryTestCase(unittest.TestCase):
         self.assertEqual(obj, outer.wrapped)
         self.assertEqual('four', outer.wrapped.two)
 
+    def test_deep_nested_subfactory(self):
+        counter = iter(range(100))
+
+        class Node(object):
+            def __init__(self, label, child=None):
+                self.id = next(counter)
+                self.label = label
+                self.child = child
+
+        class LeafFactory(factory.Factory):
+            class Meta:
+                model = Node
+            label = 'leaf'
+
+        class BranchFactory(factory.Factory):
+            class Meta:
+                model = Node
+            label = 'branch'
+            child = factory.SubFactory(LeafFactory)
+
+        class TreeFactory(factory.Factory):
+            class Meta:
+                model = Node
+            label = 'tree'
+            child = factory.SubFactory(BranchFactory)
+            child__child__label = 'magic-leaf'
+
+        leaf = LeafFactory()
+        # Magic corruption did happen here once:
+        # forcing child__child=X while another part already set another value
+        # on child__child__label meant that the value passed for child__child
+        # was merged into the factory's inner declaration dict.
+        mtree_1 = TreeFactory(child__child=leaf)
+        mtree_2 = TreeFactory()
+
+        self.assertEqual(0, mtree_1.child.child.id)
+        self.assertEqual('leaf', mtree_1.child.child.label)
+        self.assertEqual(1, mtree_1.child.id)
+        self.assertEqual('branch', mtree_1.child.label)
+        self.assertEqual(2, mtree_1.id)
+        self.assertEqual('tree', mtree_1.label)
+        self.assertEqual(3, mtree_2.child.child.id)
+        self.assertEqual('magic-leaf', mtree_2.child.child.label)
+        self.assertEqual(4, mtree_2.child.id)
+        self.assertEqual('branch', mtree_2.child.label)
+        self.assertEqual(5, mtree_2.id)
+        self.assertEqual('tree', mtree_2.label)
+
     def test_sub_factory_and_inheritance(self):
         """Test inheriting from a factory with subfactories, overriding."""
         class TestObject(object):
@@ -1545,19 +1783,19 @@ class SubFactoryTestCase(unittest.TestCase):
         class TestModelFactory(FakeModelFactory):
             class Meta:
                 model = TestModel
-            one = 3
-            two = factory.ContainerAttribute(lambda obj, containers: len(containers or []), strict=True)
+            sample_int = 3
+            container_len = factory.ContainerAttribute(lambda obj, containers: len(containers or []), strict=True)
 
         class TestModel2Factory(FakeModelFactory):
             class Meta:
                 model = TestModel2
-            one = 1
-            two = factory.SubFactory(TestModelFactory, one=1)
+            sample_int = 1
+            descendant = factory.SubFactory(TestModelFactory, sample_int=1)
 
         obj = TestModel2Factory.build()
-        self.assertEqual(1, obj.one)
-        self.assertEqual(1, obj.two.one)
-        self.assertEqual(1, obj.two.two)
+        self.assertEqual(1, obj.sample_int)
+        self.assertEqual(1, obj.descendant.sample_int)
+        self.assertEqual(1, obj.descendant.container_len)
 
         self.assertRaises(TypeError, TestModelFactory.build)
 
@@ -1568,10 +1806,10 @@ class SubFactoryTestCase(unittest.TestCase):
         class TestModelFactory(FakeModelFactory):
             class Meta:
                 model = TestModel
-            one = 3
+            sample_int = 3
 
             @factory.container_attribute
-            def two(self, containers):
+            def container_len(self, containers):
                 if containers:
                     return len(containers)
                 return 42
@@ -1579,17 +1817,17 @@ class SubFactoryTestCase(unittest.TestCase):
         class TestModel2Factory(FakeModelFactory):
             class Meta:
                 model = TestModel2
-            one = 1
-            two = factory.SubFactory(TestModelFactory, one=1)
+            sample_int = 1
+            descendant = factory.SubFactory(TestModelFactory, sample_int=1)
 
         obj = TestModel2Factory.build()
-        self.assertEqual(1, obj.one)
-        self.assertEqual(1, obj.two.one)
-        self.assertEqual(1, obj.two.two)
+        self.assertEqual(1, obj.sample_int)
+        self.assertEqual(1, obj.descendant.sample_int)
+        self.assertEqual(1, obj.descendant.container_len)
 
         obj = TestModelFactory()
-        self.assertEqual(3, obj.one)
-        self.assertEqual(42, obj.two)
+        self.assertEqual(3, obj.sample_int)
+        self.assertEqual(42, obj.container_len)
 
 
 class IteratorTestCase(unittest.TestCase):
@@ -1607,7 +1845,7 @@ class IteratorTestCase(unittest.TestCase):
             self.assertEqual(i + 10, obj.one)
 
     @unittest.skipUnless(is_python2, "Scope bleeding fixed in Python3+")
-    @tools.disable_warnings
+    @utils.disable_warnings
     def test_iterator_list_comprehension_scope_bleeding(self):
         class TestObjectFactory(factory.Factory):
             class Meta:
@@ -1619,7 +1857,7 @@ class IteratorTestCase(unittest.TestCase):
 
         self.assertRaises(TypeError, TestObjectFactory.build)
 
-    @tools.disable_warnings
+    @utils.disable_warnings
     def test_iterator_list_comprehension_protected(self):
         class TestObjectFactory(factory.Factory):
             class Meta:
@@ -1681,6 +1919,22 @@ class IteratorTestCase(unittest.TestCase):
         obj = TestObjectFactory()
         self.assertEqual(1, obj.one)
 
+    def test_iterator_time_manipulation(self):
+        class TestObjectFactory(factory.Factory):
+            class Meta:
+                model = TestObject
+
+            @factory.iterator
+            def one():
+                now = datetime.datetime.now()
+                yield now + datetime.timedelta(hours=1)
+                yield now + datetime.timedelta(hours=2)
+
+        obj1, obj2, obj3 = TestObjectFactory.create_batch(3)
+        # Timers should be t+1H, t+2H, t+1H, t+2H, etc.
+        self.assertEqual(datetime.timedelta(hours=1), obj2.one - obj1.one)
+        self.assertEqual(obj1.one, obj3.one)
+
 
 class BetterFakeModelManager(object):
     def __init__(self, keys, instance):
@@ -1959,6 +2213,38 @@ class PostGenerationTestCase(unittest.TestCase):
 
         obj = TestObjectFactory.build(bar=42, bar__foo=13)
 
+    def test_post_generation_override_with_extra(self):
+        class TestObjectFactory(factory.Factory):
+            class Meta:
+                model = TestObject
+
+            one = 1
+
+            @factory.post_generation
+            def incr_one(self, _create, override, **extra):
+                multiplier = extra.get('multiplier', 1)
+                if override is None:
+                    override = 1
+                self.one += override * multiplier
+
+        obj = TestObjectFactory.build()
+        self.assertEqual(1 + 1 * 1, obj.one)
+        obj = TestObjectFactory.build(incr_one=2)
+        self.assertEqual(1 + 2 * 1, obj.one)
+        obj = TestObjectFactory.build(incr_one__multiplier=4)
+        self.assertEqual(1 + 1 * 4, obj.one)
+        obj = TestObjectFactory.build(incr_one=2, incr_one__multiplier=5)
+        self.assertEqual(1 + 2 * 5, obj.one)
+
+        # Passing extras through inherited params
+        class OtherTestObjectFactory(TestObjectFactory):
+            class Params:
+                incr_one__multiplier = 4
+
+        obj = OtherTestObjectFactory.build()
+        self.assertEqual(1 + 1 * 4, obj.one)
+
+
     def test_post_generation_method_call(self):
         calls = []
 
@@ -1988,6 +2274,25 @@ class PostGenerationTestCase(unittest.TestCase):
         self.assertEqual(2, obj.two)
         self.assertEqual(((), {'one': 2, 'two': 3}), obj.extra)
 
+    def test_post_generation_extraction_declaration(self):
+        LIBRARY = {}
+
+        Book = collections.namedtuple('Book', ['author'])
+
+        class BookFactory(factory.Factory):
+            class Meta:
+                model = Book
+
+            author = factory.Faker('name')
+            register__reference = factory.Sequence(lambda n: n)
+
+            @factory.post_generation
+            def register(self, create, extracted, reference=0, **kwargs):
+                LIBRARY[reference] = self
+
+        book = BookFactory.build()
+        self.assertEqual({0: book}, LIBRARY)
+
     def test_related_factory(self):
         class TestRelatedObject(object):
             def __init__(self, obj=None, one=None, two=None):
@@ -2115,7 +2420,7 @@ class PostGenerationTestCase(unittest.TestCase):
                 obj.related = self
                 self.one = one
                 self.two = two
-                self.three = obj
+                self.related = obj
 
         class TestRelatedObjectFactory(factory.Factory):
             class Meta:
@@ -2133,7 +2438,7 @@ class PostGenerationTestCase(unittest.TestCase):
             one = 3
             two = 2
             three = factory.RelatedFactory(TestRelatedObjectFactory, 'obj')
-            three__two = factory.SelfAttribute('blah')
+            three__two = factory.SelfAttribute('..blah')
 
         obj = TestObjectFactory.build()
         self.assertEqual(3, obj.one)
@@ -2205,12 +2510,36 @@ class CircularTestCase(unittest.TestCase):
         self.assertIsNone(b.foo.bar.foo.bar)
 
 
+class RepeatableRandomSeedFakerTests(unittest.TestCase):
+    def test_same_seed_is_used_between_fuzzy_and_faker_generators(self):
+        class StudentFactory(factory.Factory):
+            one = factory.fuzzy.FuzzyDate(datetime.date(1950, 1, 1), )
+            two = factory.Faker('name')
+            three = factory.Faker('name', locale='it')
+            four = factory.Faker('name')
+
+            class Meta:
+                model = TestObject
+
+        seed = 1000
+        factory.random.reseed_random(seed)
+        students_1 = (StudentFactory(), StudentFactory())
+
+        factory.random.reseed_random(seed)
+        students_2 = (StudentFactory(), StudentFactory())
+
+        self.assertEqual(students_1[0].one, students_2[0].one)
+        self.assertEqual(students_1[0].two, students_2[0].two)
+        self.assertEqual(students_1[0].three, students_2[0].three)
+        self.assertEqual(students_1[0].four, students_2[0].four)
+
+
 class SelfReferentialTests(unittest.TestCase):
     def test_no_parent(self):
         from .cyclic import self_ref
 
-        obj = self_ref.TreeElementFactory(parent=None)
-        self.assertIsNone(obj.parent)
+        obj = self_ref.TreeElementFactory(parent__parent__parent=None)
+        self.assertIsNone(obj.parent.parent.parent)
 
     def test_deep(self):
         from .cyclic import self_ref
